Package: plotly
Title: Create Interactive Web Graphics via 'plotly.js'
Version: 4.12.1
Authors@R: c(person("Carson", "Sievert", role = c("aut", "cre"),
    email = "cpsievert1@gmail.com", comment = c(ORCID = "0000-0002-4958-2844")),
    person("Chris", "Parmer", role = "aut",
    email = "chris@plot.ly"),
    person("Toby", "Hocking", role = "aut",
    email = "tdhock5@gmail.com"),
    person("Scott", "Chamberlain", role = "aut",
    email = "myrmecocystus@gmail.com"),
    person("Karthik", "Ram", role = "aut",
    email = "karthik.ram@gmail.com"),
    person("Marianne", "Corvellec", role = "aut",
    email = "marianne.corvellec@igdore.org", comment = c(ORCID = "0000-0002-1994-3581")),
    person("Pedro", "Despouy", role = "aut",
    email = "pedro@plot.ly"),
    person("Salim", "Brüggemann", role = "ctb",
    email = "salim-b@pm.me", comment = c(ORCID = "0000-0002-5329-5987")),
    person("Plotly Technologies Inc.", role = "cph"))
License: MIT + file LICENSE
Description: Create interactive web graphics from 'ggplot2' graphs and/or a custom interface to the (MIT-licensed) JavaScript library 'plotly.js' inspired by the grammar of graphics.
URL: https://plotly-r.com, https://github.com/plotly/plotly.R,
        https://plotly.com/r/
BugReports: https://github.com/plotly/plotly.R/issues
Depends: R (>= 3.5.0), ggplot2 (>= 3.0.0)
Imports: tools, scales, httr (>= 1.3.0), jsonlite (>= 1.6), magrittr,
        digest, viridisLite, base64enc, htmltools (>= 0.3.6),
        htmlwidgets (>= 1.5.2.9001), tidyr (>= 1.0.0), RColorBrewer,
        dplyr, vctrs, tibble, rlang (>= 1.0.0), crosstalk, purrr,
        data.table, promises
Suggests: MASS, maps, hexbin, ggthemes, GGally, ggalluvial, testthat,
        knitr, shiny (>= 1.1.0), shinytest2, curl, rmarkdown, Cairo,
        broom, webshot, listviewer, dendextend, sf, png, IRdisplay,
        processx, plotlyGeoAssets, forcats, withr, palmerpenguins,
        rversions, reticulate, rsvg, ggridges
LazyData: true
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/Needs/check: tidyverse/ggplot2, ggobi/GGally, rcmdcheck,
        devtools, reshape2, s2
NeedsCompilation: no
Packaged: 2026-07-21 23:17:16 UTC; cpsievert
Author: Carson Sievert [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4958-2844>),
  Chris Parmer [aut],
  Toby Hocking [aut],
  Scott Chamberlain [aut],
  Karthik Ram [aut],
  Marianne Corvellec [aut] (ORCID:
    <https://orcid.org/0000-0002-1994-3581>),
  Pedro Despouy [aut],
  Salim Brüggemann [ctb] (ORCID: <https://orcid.org/0000-0002-5329-5987>),
  Plotly Technologies Inc. [cph]
Maintainer: Carson Sievert <cpsievert1@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-22 06:10:02 UTC
