# Generated by roxygen2: do not edit by hand

S3method(api_create,data.frame)
S3method(api_create,ggplot)
S3method(api_create,plotly)
S3method(embed_notebook,plotly)
S3method(fortify,SharedData)
S3method(geom2trace,GeomBar)
S3method(geom2trace,GeomBlank)
S3method(geom2trace,GeomBoxplot)
S3method(geom2trace,GeomErrorbar)
S3method(geom2trace,GeomErrorbarh)
S3method(geom2trace,GeomPath)
S3method(geom2trace,GeomPoint)
S3method(geom2trace,GeomPolygon)
S3method(geom2trace,GeomRidgelineGradient)
S3method(geom2trace,GeomText)
S3method(geom2trace,GeomTile)
S3method(geom2trace,default)
S3method(ggplot,plotly)
S3method(ggplotly,"NULL")
S3method(ggplotly,ggmatrix)
S3method(ggplotly,ggplot)
S3method(ggplotly,plotly)
S3method(highlight_key,default)
S3method(highlight_key,plotly)
S3method(layout,matrix)
S3method(layout,plotly)
S3method(layout,shiny.tag.list)
S3method(linewidth_or_size,Geom)
S3method(linewidth_or_size,default)
S3method(linewidth_or_size,element)
S3method(plotly_build,"NULL")
S3method(plotly_build,gg)
S3method(plotly_build,ggmatrix)
S3method(plotly_build,list)
S3method(plotly_build,plotly)
S3method(print,api)
S3method(print,api_grid)
S3method(print,api_grid_local)
S3method(print,api_plot)
S3method(print,kaleidoScope)
S3method(print,plotly_data)
S3method(process,api_image)
S3method(process,api_plot)
S3method(process,default)
S3method(to_basic,GeomAbline)
S3method(to_basic,GeomAlluvium)
S3method(to_basic,GeomAnnotationMap)
S3method(to_basic,GeomArea)
S3method(to_basic,GeomBoxplot)
S3method(to_basic,GeomCol)
S3method(to_basic,GeomContour)
S3method(to_basic,GeomCrossbar)
S3method(to_basic,GeomDensity)
S3method(to_basic,GeomDensity2d)
S3method(to_basic,GeomDensityLine)
S3method(to_basic,GeomDensityRidges)
S3method(to_basic,GeomDensityRidges2)
S3method(to_basic,GeomDensityRidgesGradient)
S3method(to_basic,GeomDotplot)
S3method(to_basic,GeomErrorbar)
S3method(to_basic,GeomErrorbarh)
S3method(to_basic,GeomFunction)
S3method(to_basic,GeomHex)
S3method(to_basic,GeomHline)
S3method(to_basic,GeomJitter)
S3method(to_basic,GeomLine)
S3method(to_basic,GeomLinerange)
S3method(to_basic,GeomMap)
S3method(to_basic,GeomPointrange)
S3method(to_basic,GeomQuantile)
S3method(to_basic,GeomRaster)
S3method(to_basic,GeomRasterAnn)
S3method(to_basic,GeomRect)
S3method(to_basic,GeomRibbon)
S3method(to_basic,GeomRidgeline)
S3method(to_basic,GeomRidgelineGradient)
S3method(to_basic,GeomRug)
S3method(to_basic,GeomSegment)
S3method(to_basic,GeomSf)
S3method(to_basic,GeomSmooth)
S3method(to_basic,GeomSpoke)
S3method(to_basic,GeomStep)
S3method(to_basic,GeomStratum)
S3method(to_basic,GeomTile)
S3method(to_basic,GeomViolin)
S3method(to_basic,GeomVline)
S3method(to_basic,default)
export("%>%")
export(TeX)
export(add_annotations)
export(add_area)
export(add_bars)
export(add_boxplot)
export(add_choropleth)
export(add_contour)
export(add_data)
export(add_fun)
export(add_heatmap)
export(add_histogram)
export(add_histogram2d)
export(add_histogram2dcontour)
export(add_image)
export(add_lines)
export(add_markers)
export(add_mesh)
export(add_paths)
export(add_pie)
export(add_polygons)
export(add_ribbons)
export(add_scattergeo)
export(add_segments)
export(add_sf)
export(add_surface)
export(add_table)
export(add_text)
export(add_trace)
export(animation_button)
export(animation_opts)
export(animation_slider)
export(api)
export(api_create)
export(api_download_grid)
export(api_download_plot)
export(arrange)
export(arrange_)
export(as.widget)
export(as_widget)
export(attrs_selected)
export(colorbar)
export(config)
export(distinct)
export(distinct_)
export(do)
export(do_)
export(embed_notebook)
export(event_data)
export(event_register)
export(event_unregister)
export(export)
export(filter)
export(filter_)
export(geom2trace)
export(get_figure)
export(gg2list)
export(ggplotly)
export(group2NA)
export(group_by)
export(group_by_)
export(groups)
export(hide_colorbar)
export(hide_guides)
export(hide_legend)
export(highlight)
export(highlight_key)
export(kaleido)
export(knit_print.api_grid)
export(knit_print.api_grid_local)
export(knit_print.api_plot)
export(last_plot)
export(layout)
export(mutate)
export(mutate_)
export(offline)
export(orca)
export(orca_serve)
export(partial_bundle)
export(plot_dendro)
export(plot_geo)
export(plot_ly)
export(plot_mapbox)
export(plotly)
export(plotlyOutput)
export(plotlyProxy)
export(plotlyProxyInvoke)
export(plotly_IMAGE)
export(plotly_POST)
export(plotly_build)
export(plotly_data)
export(plotly_empty)
export(plotly_example)
export(plotly_json)
export(rangeslider)
export(raster2uri)
export(remove_typedarray_polyfill)
export(rename)
export(rename_)
export(renderPlotly)
export(save_image)
export(schema)
export(select)
export(select_)
export(showRGB)
export(signup)
export(slice)
export(slice_)
export(style)
export(subplot)
export(summarise)
export(summarise_)
export(toRGB)
export(toWebGL)
export(to_basic)
export(transmute)
export(transmute_)
export(ungroup)
import(ggplot2)
importFrom(data.table,as.data.table)
importFrom(data.table,setorderv)
importFrom(dplyr,arrange)
importFrom(dplyr,arrange_)
importFrom(dplyr,distinct)
importFrom(dplyr,distinct_)
importFrom(dplyr,do)
importFrom(dplyr,do_)
importFrom(dplyr,filter)
importFrom(dplyr,filter_)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_)
importFrom(dplyr,groups)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_)
importFrom(dplyr,rename)
importFrom(dplyr,rename_)
importFrom(dplyr,select)
importFrom(dplyr,select_)
importFrom(dplyr,slice)
importFrom(dplyr,slice_)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_)
importFrom(dplyr,transmute)
importFrom(dplyr,transmute_)
importFrom(dplyr,ungroup)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,extendrange)
importFrom(grDevices,rgb)
importFrom(graphics,layout)
importFrom(htmltools,browsable)
importFrom(htmltools,div)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(htmlwidgets,createWidget)
importFrom(htmlwidgets,onRender)
importFrom(htmlwidgets,prependContent)
importFrom(htmlwidgets,saveWidget)
importFrom(htmlwidgets,shinyRenderWidget)
importFrom(htmlwidgets,shinyWidgetOutput)
importFrom(htmlwidgets,sizingPolicy)
importFrom(httr,RETRY)
importFrom(httr,add_headers)
importFrom(httr,authenticate)
importFrom(httr,config)
importFrom(httr,content)
importFrom(httr,stop_for_status)
importFrom(httr,warn_for_status)
importFrom(httr,write_disk)
importFrom(jsonlite,parse_json)
importFrom(jsonlite,read_json)
importFrom(jsonlite,toJSON)
importFrom(lazyeval,all_dots)
importFrom(lazyeval,f_eval)
importFrom(lazyeval,f_new)
importFrom(lazyeval,is_formula)
importFrom(lazyeval,is_lang)
importFrom(magrittr,"%>%")
importFrom(purrr,transpose)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,eval_tidy)
importFrom(rlang,is_na)
importFrom(stats,complete.cases)
importFrom(stats,is.leaf)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(tibble,as_tibble)
importFrom(tidyr,unnest)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,file.edit)
importFrom(utils,getFromNamespace)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,str)
importFrom(viridisLite,viridis)
