#' Wind data
#'
#' Description TBD.
#'
#' @format A data frame with three variables: `r`, `t`,
#'   `nms`.
"wind"

#' Mic data
#'
#' Description TBD.
#'
#' @format A data frame with three variables: `r`, `t`,
#'   `nms`.
"mic"

#' Hobbs data
#'
#' Description TBD.
#'
#' @format A data frame with three variables: `r`, `t`,
#'   `nms`.
"hobbs"


#' Minnesotan Indian Reservation Lands
#'
#' @references \url{https://www.dot.state.mn.us/maps/gdma/gis-data.html}
#'
#' @format An sf data frame with 13 features and 5 fields
#' 
"res_mn"
