library(shinytest)
app <- ShinyDriver$new("../../", shinyOptions = list(display.mode = "normal"))
app$snapshotInit("mytest")

app$snapshot()
app$setInputs(`plotly_hover-A` = "[{\"curveNumber\":0,\"pointNumber\":7,\"x\":24.4,\"y\":3.19,\"customdata\":\"Merc 240D\"}]", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_click-A` = "[{\"curveNumber\":0,\"pointNumber\":7,\"x\":24.4,\"y\":3.19,\"customdata\":\"Merc 240D\"}]", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_hover-A` = character(0), allowInputNoBinding_ = TRUE)
Sys.sleep(1)
app$snapshot()
app$setInputs(`plotly_brushing-A` = "{\"x\":[25.726819184123485,25.98332414553473],\"y\":[1.3174499999999998,5.61955]}", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_selecting-A` = "[]", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_brushing-A` = "{\"x\":[24.64379823594267,25.98332414553473],\"y\":[3.2093373493975905,3.5073743975903615]}", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_brushing-A` = "{\"x\":[24.045286659316428,25.98332414553473],\"y\":[3.040881626506024,3.5073743975903615]}", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_selecting-A` = "[{\"curveNumber\":0,\"pointNumber\":7,\"x\":24.4,\"y\":3.19,\"customdata\":\"Merc 240D\"}]", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_brushing-A` = "{\"x\":[23.95978500551268,25.98332414553473],\"y\":[3.0020072289156627,3.5073743975903615]}", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_selected-A` = "[{\"curveNumber\":0,\"pointNumber\":7,\"x\":24.4,\"y\":3.19,\"customdata\":\"Merc 240D\"}]", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_brushed-A` = "{\"x\":[23.95978500551268,25.98332414553473],\"y\":[3.0020072289156627,3.5073743975903615]}", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_hover-A` = "[{\"curveNumber\":0,\"pointNumber\":7,\"x\":24.4,\"y\":3.19,\"customdata\":\"Merc 240D\"}]", allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_hover-A` = character(0), allowInputNoBinding_ = TRUE)
app$snapshot()
app$setInputs(`plotly_selected-A` = character(0), allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_brushed-A` = character(0), allowInputNoBinding_ = TRUE)
#app$setInputs(`plotly_selected-A` = character(0), allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_selecting-A` = character(0), allowInputNoBinding_ = TRUE)
#app$setInputs(`plotly_brushed-A` = character(0), allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_brushing-A` = character(0), allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_click-A` = character(0), allowInputNoBinding_ = TRUE)
app$setInputs(`plotly_deselect-A` = "\"plot\"", allowInputNoBinding_ = TRUE)
app$snapshot()
