

set -ex



pip check
python -c "import plotly.matplotlylib; import plotly.matplotlylib.mplexporter; import plotly.matplotlylib.mplexporter.renderers"
python -c "import plotly.figure_factory"
python -c "import plotly.express; import plotly.express.data; import plotly.express.colors; import plotly.express.trendline_functions"
exit 0
