% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{knit_print.api_grid_local}
\alias{knit_print.api_grid_local}
\title{Embed a plotly grid as an iframe in a knitr doc}
\usage{
knit_print.api_grid_local(x, options, ...)
}
\arguments{
\item{x}{a plotly figure object}

\item{options}{knitr options.}

\item{...}{placeholder.}
}
\description{
Embed a plotly grid as an iframe in a knitr doc
}
\references{
https://github.com/yihui/knitr/blob/master/vignettes/knit_print.Rmd
}
