% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/embed.R
\name{embed_notebook}
\alias{embed_notebook}
\title{Embed a plot as an iframe into a Jupyter Notebook}
\usage{
embed_notebook(x, width = NULL, height = NULL, file = NULL)
}
\arguments{
\item{x}{a plotly object}

\item{width}{attribute of the iframe. If \code{NULL}, the width in
\code{plot_ly} is used. If that is also \code{NULL}, '100\%' is the default.}

\item{height}{attribute of the iframe. If \code{NULL}, the height in
\code{plot_ly} is used. If that is also \code{NULL}, '400px' is the default.}

\item{file}{deprecated.}
}
\description{
Embed a plot as an iframe into a Jupyter Notebook
}
\author{
Carson Sievert
}
