% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mic}
\alias{mic}
\title{Mic data}
\format{
A data frame with three variables: \code{r}, \code{t},
\code{nms}.
}
\usage{
mic
}
\description{
Description TBD.
}
\keyword{datasets}
